package models

import (
	"strings"
	"time"

	"github.com/jackc/pgtype"
	"orus.io/orus-io/go-orusapi/database"
)

// dbtable:"email_queue" dbpkey:"id"
type Email struct {
	ID            int              `db:"id"`
	MessageID     string           `db:"message_id"`
	CreatedAt     time.Time        `db:"created_at"`
	From          string           `db:"from_"`
	To            pgtype.TextArray `db:"to_"`
	Msg           []byte           `db:"msg"`
	StatusChanged time.Time        `db:"status_changed"`
	Status        string           `db:"status"`
	LastError     string           `db:"last_error"`
	Attempts      int              `db:"attempts"`
	ReattemptAt   time.Time        `db:"reattempt_at"`
}

func QueueMail(
	db database.SQLHelper, messageID string,
	from string, to []string,
	headers Metadata,
	msg []byte,
) error {
	sHeaders := "Message-ID: " + messageID + "\n"

	if len(headers) != 0 {
		for k, v := range headers {
			if strings.ToLower(k) != "message-id" {
				sHeaders += k + ": " + v + "\n"
			}
		}
	}

	sHeaders += "\n"

	msg = append([]byte(sHeaders), msg...)

	email := Email{
		MessageID:     messageID,
		CreatedAt:     time.Now().UTC(),
		From:          from,
		Msg:           msg,
		Status:        "pending",
		StatusChanged: time.Now().UTC(),
	}
	if err := email.To.Set(to); err != nil {
		return err
	}

	if _, err := db.InsertNoPKey(&email); err != nil {
		return err
	}

	if _, err := db.ExecRaw("NOTIFY email_queue_notify"); err != nil {
		return err
	}

	return nil
}
